// Copyright (c) Microsoft. All rights reserved.
/// <reference path="c:\Program Files (x86)\Microsoft Visual Studio 14.0\Common7\ide\Extensions\Microsoft\Node.js Tools for Visual Studio\1.1\Scripts\Typings\Node\node.d.ts"/>
/// <reference path="y:\dev\projects\AdaFruit\ts\winrt.d.ts"/>
/// <reference path="y:\dev\projects\AdaFruit\ts\winuwp.d.ts"/>
// Want to do OAuth in the future
// OAuth2 http://scottksmith.com/blog/2014/07/02/beer-locker-building-a-restful-api-with-node-oauth2-server/
// http://resources.infosecinstitute.com/securing-web-apis-part-ii-creating-an-api-authenticated-with-oauth-2-in-node-js/
var http = require('http');
var https = require('https');
var url = require('url');
var path = require('path');
var fs = require('fs');
process.env.NODE_PATH = "./npm"; // node_modules";
var ws = require("nodejs-websocket"); // https://github.com/sitegui/nodejs-websocket
var uwp = require("uwp");
function log(msg) {
    console.log(msg); // Do we even have a console?
}
var blinky;
(function (blinky) {
    var WebConnection = (function () {
        function WebConnection(conn) {
            var _this = this;
            this.conn = conn;
            this.cid = WebConnection.connectionCount++;
            this.socket = conn.socket;
            conn.sendText("#" + this.cid + "/" + WebConnection.connections.length + " Hello!");
            var adr = this.socket.address();
            WebConnection.WSReport("#" + this.cid + " ws from " + this.socket.remoteAddress + " " + this.socket.remotePort + " to " + adr.address + " " + adr.port);
            conn.on("text", function (str) {
                log("Received " + str);
                if (str[0] == "{") {
                    Bulb.LEDRequest(JSON.parse(str), _this);
                }
                else if (str == "reload!")
                    WebConnection.WSReport(str);
                else
                    conn.sendText("Echo[" + str.toUpperCase() + "]");
            });
            conn.on("close", (function (code, reason) {
                console.log("Connection closed");
                WebConnection.connections = WebConnection.connections.filter(function (wc) { return wc.cid != _this.cid; });
            }).bind(this));
            conn.on("error", (function (err) {
                var rmsg = _this.socket.remoteAddress + " " + _this.socket.remotePort;
                if (rmsg.indexOf("undefined") >= 0)
                    rmsg = "";
                WebConnection.WSReport("#" + _this.cid + "/" + WebConnection.connections.length + " " + rmsg + " " + err);
            }).bind(this));
        }
        WebConnection.prototype.report = function (msg) {
            try {
                this.conn.sendText(msg);
            }
            catch (e) {
            }
        };
        WebConnection.WSReport = function (msg) {
            if (!WebConnection || !WebConnection.connections)
                return; // Too early
            var txt = typeof msg == "string" ? msg : JSON.stringify(msg);
            WebConnection.connections.forEach(function (wc) {
                try {
                    wc.report(txt);
                }
                catch (e) { }
                ;
            });
        };
        WebConnection.ReportBulbs = function () {
            WebConnection.WSReport({
                Red: Bulb.red ? Bulb.red.isOn : null,
                Green: Bulb.green ? Bulb.green.isOn : null,
                CName: process.env.COMPUTERNAME,
                Count: Bulb.blinks,
                Interval: Bulb.flipInterval,
                OnPeriod: Bulb.flipDelay,
            });
        };
        WebConnection.handleWS = function (conn) {
            try {
                log("New connection");
                WebConnection.connections.push(new WebConnection(conn));
            }
            catch (e) {
                log("WS Error " + e);
            }
        };
        WebConnection.SetupWS = function () {
            try {
                log("Setting up WS");
                // Scream server example: "hi" -> "HI!!!"
                var server = ws.createServer(WebConnection.handleWS);
                server.listen(8003);
                options["secure"] = true; // Wss seems to need this
                var server2 = ws.createServer(options, WebConnection.handleWS);
                server2.listen(8004);
            }
            catch (e) {
                log("SetupWS " + e);
            }
        };
        WebConnection.connections = [];
        WebConnection.connectionCount = 0;
        return WebConnection;
    })();
    var Site = (function () {
        function Site(port, secure) {
            // original was much simpler
            //var server = http.createServer((req, resp) => {
            //    resp.writeHead(200, { "Content-Type": "Text/plain" });
            //    resp.end(`Got LED ${currentValue.toString() }`);
            //});
            this.favicon = '/favicon.ico'; // So we can ignore it
            // TODO: Port Mapping https://gist.github.com/smolleyes/a3dea6e5086dd4d064f6
            var server = secure ? https.createServer(options, this.ServerResponder.bind(this)) : http.createServer(this.ServerResponder.bind(this));
            server.listen(port);
        }
        Site.prototype.ServerResponder = function (request, response) {
            var body = "";
            var up = url.parse(request.url, true);
            var qd = up.query;
            var red = qd.red == "on" ? true : qd.red == "off" ? false : null;
            var green = qd.green == "on" ? true : qd.green == "off" ? false : null;
            var conn = request.connection;
            var adr = conn.address();
            if (up.path == "/")
                WebConnection.WSReport("http request " + conn.remoteAddress + ":" + conn.remotePort + " to " + adr.address + ":" + adr.port + "\n" + request.url);
            var bm = { Red: red, Green: green, Interval: Bulb.flipInterval };
            request.on("data", function (data) {
                body += data;
            });
            request.on('end', function () {
                // Process the URL here
                var fpaths = up.path.split('/');
                var fname = fpaths[fpaths.length - 1]; // Get the last component
                if (fname == "")
                    fname = "Client.html"; // Default
                var ext = path.extname(fname);
                try {
                    var mime = {
                        ".js": "application/javascript",
                        ".css": "text/css",
                        ".html": "text/html",
                        ".ico": "*image/x-icon",
                        ".png": "*image/png",
                        ".jpg": "*image/jpeg"
                    };
                    var kind = mime[ext];
                    if (!kind)
                        return; // { // We have a hit
                    if (kind[0] == "*") {
                        var img = fs.readFileSync(path.join(assetsPath, fname)); // http://convertico.com/
                        response.writeHead(200, { 'content-type': kind.substr(1) });
                        response.end(img, 'binary');
                    }
                    else {
                        var js = fs.readFileSync(path.join(webDir, fname), "utf8");
                        response.writeHead(200, { 'content-type': kind }); // Content-type not always necessary
                        response.end(js);
                    }
                    return;
                }
                catch (e) {
                    response.writeHead(404, { 'content-type': 'text/plain' });
                    response.end("error " + e);
                }
            });
        };
        Site.Init = function () {
        };
        return Site;
    })();
    var Bulb = (function () {
        function Bulb(pinNumber) {
            this.pin = Bulb.gpio.openPin(pinNumber);
            this.pin.setDriveMode(Windows.Devices.Gpio.GpioPinDriveMode.output);
            this.Set(true); // Turn both on to start
        }
        Bulb.prototype.Set = function (on) {
            if (on == null)
                return;
            this.isOn = on;
            this.pin.write(this.isOn ? Windows.Devices.Gpio.GpioPinValue.low : Windows.Devices.Gpio.GpioPinValue.high);
            WebConnection.ReportBulbs();
        };
        Bulb.Init = function () {
            this.gpio = Windows.Devices.Gpio.GpioController.getDefault();
            Bulb.red = new Bulb(5);
            Bulb.green = new Bulb(6);
        };
        Bulb.LEDRequest = function (got, wc) {
            WebConnection.WSReport("Got request #" + wc.cid + " " + JSON.stringify(got).replace("{", "[").replace("}", "]")); // Program around JSON kludge for text
            Bulb.red.Set(got.Red);
            Bulb.green.Set(got.Green);
            if (got.Interval) {
                var ival = got.Interval;
                if (ival >= .5 && ival <= 5.0) {
                    Bulb.flipInterval = ival;
                    WebConnection.ReportBulbs(); // Report interval change
                }
            }
            if (got.Red || got.Green)
                Bulb.flipCount = Bulb.flipDelay / Bulb.flipInterval;
        };
        Bulb.flipLed = function () {
            if (--Bulb.flipCount <= 0) {
                Bulb.blinks++;
                Bulb.flipValue = !Bulb.flipValue;
                Bulb.red.Set(Bulb.flipValue);
                Bulb.green.Set(!Bulb.flipValue);
            }
            setTimeout(Bulb.flipLed, Bulb.flipInterval * 1000);
        };
        Bulb.blinks = 0;
        Bulb.flipCount = 0;
        Bulb.flipInterval = 1; // Seconds
        Bulb.flipDelay = 10;
        return Bulb;
    })();
    try {
        log("Starting Blinky Server");
        uwp.projectNamespace("Windows");
        Bulb.Init();
        {
            var assetsPath = path.join(__dirname, "Assets");
            var webDir = path.join(__dirname, "Web");
            var certsPath = path.join(__dirname, 'certs', 'server');
            //let caCertsPath = path.join(__dirname, 'certs', 'ca');
            log("certsPath: " + certsPath);
            var options = {
                key: fs.readFileSync(path.join(certsPath, 'my-server.key.pem')),
                cert: fs.readFileSync(path.join(certsPath, 'my-server.crt.pem')),
                requestCert: false,
                rejectUnauthorized: true
            };
        }
        Site.Init();
        new Site(8001, false);
        new Site(8002, true);
        WebConnection.SetupWS();
        setTimeout(Bulb.flipLed, 25);
    }
    catch (e) {
        debugger;
    }
})(blinky || (blinky = {}));
//# sourceMappingURL=Blinker.js.map