﻿// Copyright (c) Microsoft. All rights reserved.
/// <reference path="c:\Program Files (x86)\Microsoft Visual Studio 14.0\Common7\ide\Extensions\Microsoft\Node.js Tools for Visual Studio\1.1\Scripts\Typings\Node\node.d.ts"/>
/// <reference path="y:\dev\projects\AdaFruit\ts\winrt.d.ts"/>
/// <reference path="y:\dev\projects\AdaFruit\ts\winuwp.d.ts"/>

// Want to do OAuth in the future
// OAuth2 http://scottksmith.com/blog/2014/07/02/beer-locker-building-a-restful-api-with-node-oauth2-server/
// http://resources.infosecinstitute.com/securing-web-apis-part-ii-creating-an-api-authenticated-with-oauth-2-in-node-js/

import http = require('http');
import https = require('https');
import url = require('url');
import path = require('path');
import net = require('net');
import fs = require('fs');
process.env.NODE_PATH = "./npm"; // node_modules";
var ws = require("nodejs-websocket");                  // https://github.com/sitegui/nodejs-websocket
var uwp = require("uwp");

function log(msg: string) {
    console.log(msg);   // Do we even have a console?
}

module blinky {
    class WebConnection {
        constructor(public conn) {
            this.cid = WebConnection.connectionCount++;
            this.socket = conn.socket;
            conn.sendText(`#${this.cid}/${WebConnection.connections.length} Hello!`);
            let adr = this.socket.address();
            WebConnection.WSReport(`#${this.cid} ws from ${this.socket.remoteAddress} ${this.socket.remotePort} to ${adr.address} ${adr.port}`);
            conn.on("text", (str: string) => {
                log(`Received ${str}`);
                if (str[0] == "{") { // Then this is a request
                    Bulb.LEDRequest(JSON.parse(str), this);
                }
                else if (str == "reload!")
                    WebConnection.WSReport(str);
                else
                    conn.sendText(`Echo[${str.toUpperCase()}]`);
            });
            conn.on("close", ((code, reason) => {
                console.log("Connection closed")
                WebConnection.connections = WebConnection.connections.filter(wc => wc.cid != this.cid);
            }).bind(this));
            conn.on("error", ((err) => {
                let rmsg = `${this.socket.remoteAddress} ${this.socket.remotePort}`;
                if (rmsg.indexOf("undefined") >= 0) rmsg = "";
                WebConnection.WSReport(`#${this.cid}/${WebConnection.connections.length} ${rmsg} ${err}`);
            }).bind(this));
        }

        cid: number;
        socket: net.Socket;
        static connections: WebConnection[] = [];
        static connectionCount = 0;

        public report(msg: string) {
            try {
                this.conn.sendText(msg);
            }
            catch (e) {
            }
        }

        static WSReport(msg: string | BlinkReport) {    // Send message to all listeners
            if (!WebConnection || !WebConnection.connections) return;     // Too early
            let txt = typeof msg == "string" ? <string>msg : JSON.stringify(msg);
            WebConnection.connections.forEach((wc) => {
                try {
                    wc.report(txt);
                }
                catch (e) { };
            });
        }

        static ReportBulbs() {
            WebConnection.WSReport({
                Red: Bulb.red ? Bulb.red.isOn : null,
                Green: Bulb.green ? Bulb.green.isOn : null,
                CName: process.env.COMPUTERNAME,
                Count: Bulb.blinks,
                Interval: Bulb.flipInterval,
                OnPeriod: Bulb.flipDelay,
            });
        }

        static handleWS(conn) {
            try {
                log("New connection")
                WebConnection.connections.push(new WebConnection(conn));
            }
            catch (e) {
                log(`WS Error ${e}`);
            }
        }

        static SetupWS() {
            try {
                log("Setting up WS");
                // Scream server example: "hi" -> "HI!!!"
                let server = ws.createServer(WebConnection.handleWS);
                server.listen(8003)

                options["secure"] = true;   // Wss seems to need this
                let server2 = ws.createServer(options, WebConnection.handleWS);
                server2.listen(8004);
            }
            catch (e) {
                log(`SetupWS ${e}`);
            }
        }
    }

    class Site {
        favicon = '/favicon.ico';   // So we can ignore it

        ServerResponder(request: http.ServerRequest, response: http.ServerResponse) {
            let body = "";
            let up = url.parse(request.url, true);
            let qd = up.query;
            let red = qd.red == "on" ? true : qd.red == "off" ? false : null;
            let green = qd.green == "on" ? true : qd.green == "off" ? false : null;
            let conn = request.connection;
            let adr = conn.address();
            if (up.path == "/") WebConnection.WSReport(`http request ${conn.remoteAddress}:${conn.remotePort} to ${adr.address}:${adr.port}\n${request.url}`);
            let bm: BlinkMsgV2 = { Red: red, Green: green, Interval: Bulb.flipInterval };

            request.on("data", (data) => {
                body += data;
            });

            request.on('end', () => {
                // Process the URL here
                let fpaths = up.path.split('/');
                let fname = fpaths[fpaths.length - 1];  // Get the last component
                if (fname == "") fname = "Client.html"; // Default
                let ext = path.extname(fname);
                try {
                    let mime = {
                        ".js": "application/javascript",
                        ".css": "text/css",
                        ".html": "text/html",
                        ".ico": "*image/x-icon",
                        ".png": "*image/png",
                        ".jpg": "*image/jpeg"
                    };

                    let kind = mime[ext];
                    if (!kind) return; // { // We have a hit
                    if (kind[0] == "*") {   // Binary 
                        let img = fs.readFileSync(path.join(assetsPath, fname)); // http://convertico.com/
                        response.writeHead(200, { 'content-type': kind.substr(1) });
                        response.end(img, 'binary');
                    }
                    else {
                        let js = fs.readFileSync(path.join(webDir, fname), "utf8");
                        response.writeHead(200, { 'content-type': kind });  // Content-type not always necessary
                        response.end(js);
                    }
                    return;
                }
                catch (e) {
                    response.writeHead(404, { 'content-type': 'text/plain' });
                    response.end(`error ${e}`);
                }
            });
        }

        constructor(port: number, secure: boolean) {
            // original was much simpler
            //var server = http.createServer((req, resp) => {
            //    resp.writeHead(200, { "Content-Type": "Text/plain" });
            //    resp.end(`Got LED ${currentValue.toString() }`);
            //});

            // TODO: Port Mapping https://gist.github.com/smolleyes/a3dea6e5086dd4d064f6

            let server = secure ? https.createServer(options, this.ServerResponder.bind(this)) : http.createServer(this.ServerResponder.bind(this));
            server.listen(port);
        }

        static Init() {
        }

        //static images: string;
    }

    class Bulb {
        constructor(pinNumber: number) {
            this.pin = Bulb.gpio.openPin(pinNumber);
            this.pin.setDriveMode(Windows.Devices.Gpio.GpioPinDriveMode.output);
            this.Set(true);     // Turn both on to start
        }
        pin: Windows.Devices.Gpio.GpioPin;
        isOn: boolean;

        Set(on: boolean) {
            if (on == null) return;
            this.isOn = on;
            this.pin.write(this.isOn ? Windows.Devices.Gpio.GpioPinValue.low : Windows.Devices.Gpio.GpioPinValue.high);
            WebConnection.ReportBulbs();
        }

        static Init() {
            this.gpio = Windows.Devices.Gpio.GpioController.getDefault();
            Bulb.red = new Bulb(5);
            Bulb.green = new Bulb(6);
        }

        static gpio: Windows.Devices.Gpio.GpioController;
        static red: Bulb;
        static green: Bulb;
        static blinks: number = 0;
        static flipCount: number = 0;
        static flipInterval: number = 1;    // Seconds
        static flipDelay: number = 10;
        static flipValue: boolean;

        static LEDRequest(got: BlinkMsgV2, wc: WebConnection) {
            WebConnection.WSReport(`Got request #${wc.cid} ${JSON.stringify(got).replace("{", "[").replace("}", "]")}`);   // Program around JSON kludge for text
            Bulb.red.Set(got.Red);
            Bulb.green.Set(got.Green);
            if (got.Interval) {
                let ival = got.Interval;
                if (ival >= .5 && ival <= 5.0) {
                    Bulb.flipInterval = ival;
                    WebConnection.ReportBulbs();    // Report interval change
                }
            }
            if (got.Red || got.Green)   // If we got a change
                Bulb.flipCount = Bulb.flipDelay / Bulb.flipInterval;
        }

        static flipLed() {
            if (--Bulb.flipCount <= 0) {
                Bulb.blinks++;
                Bulb.flipValue = !Bulb.flipValue;
                Bulb.red.Set(Bulb.flipValue);
                Bulb.green.Set(!Bulb.flipValue);
            }
            setTimeout(Bulb.flipLed, Bulb.flipInterval * 1000);
        }
    }

    try {   // Initialize site and bulbs
        log("Starting Blinky Server");
        uwp.projectNamespace("Windows");
        Bulb.Init();

        {    // Initialize common web stuff shared for HTTP and WS
            var assetsPath = path.join(__dirname, "Assets")

            var webDir = path.join(__dirname, "Web");
            let certsPath = path.join(__dirname, 'certs', 'server');
            //let caCertsPath = path.join(__dirname, 'certs', 'ca');
            log(`certsPath: ${certsPath}`);
            var options: https.ServerOptions = {
                key: fs.readFileSync(path.join(certsPath, 'my-server.key.pem')),    // Private
                cert: fs.readFileSync(path.join(certsPath, 'my-server.crt.pem')),   // Public
                requestCert: false,
                rejectUnauthorized: true
            };
        }

        Site.Init();
        new Site(8001, false);
        new Site(8002, true);
        WebConnection.SetupWS();

        setTimeout(Bulb.flipLed, 25);
    }
    catch (e) {
        debugger;
    }
}
