﻿enum ReadyState {
    Unsent = 0,             // 0	UNSENT	open() has not been called yet.
    Opened = 1,             // 1	OPENED	send() has not been called yet.
    HeadersReceived = 2,    // 2	HEADERS_RECEIVED	send() has been called, and headers and status are available.
    Loading = 3,            // 3	LOADING	Downloading; responseText holds partial data.   
    Ready = 4               // 4	DONE	The operation is complete.
}

module blinks {
    let logd: HTMLDivElement;

    export function BlinkInit() {
        try {
            logd = <HTMLDivElement>document.getElementById("logdiv");

            let buttons = document.getElementById("buttons");

            //var btn = (text: string, color: string) => {
            //    let btn = document.createElement("button");
            //    btn.textContent = text;
            //    btn.style.backgroundColor = color;
            //    btn.style.fontSize = "14pt";
            //    btn.style.width = ".75in";
            //    btn.style.textAlign = "center";
            //    buttons.appendChild(btn);
            //    return btn;
            //};

            let btn = (id: string, color: string) => {
                let btn = <HTMLButtonElement>document.getElementById(id);
                btn.classList.add('btn');
                btn.style.backgroundColor = color;
                return btn;
            };

            redInfo = new bulbInfo("red", btn("red", "crimson"), "crimson", "darkred");
            redInfo.btn.addEventListener("click", () => Blink({ Red: true }));

            greenInfo = new bulbInfo("green", btn("green", "lime"), "lime", "DarkGreen");
            greenInfo.btn.addEventListener("click", () => Blink({ Green: true }));

            let btnReload = btn("reload", "lightGrey");
            btnReload.style.color = "Red";
            btnReload.addEventListener("click", () => ws.send("reload!"));

            spanCount = document.createElement("span");
            spanCount.style.fontSize = "14pt";
            buttons.appendChild(spanCount);

            spanPeriod = <HTMLSpanElement>document.getElementById("OnPeriod");

            buttons.appendChild(document.createTextNode(" Interval: "));
            intervalInput = document.createElement("input");
            intervalInput.type = "range";
            intervalInput.min = ".5";
            intervalInput.max = "3";
            intervalInput.step = ".5";
            buttons.appendChild(intervalInput);

            intval = document.createElement("span");
            buttons.appendChild(intval);
            intervalInput.addEventListener("change", () => {
                intval.textContent = intervalInput.value;
                intval.style.backgroundColor = "pink";
                Blink({});  // Send new interval
            });
            intval.textContent = intervalInput.value;

            log("Initialized");
            wsTest();
        }
        catch (e) {
            alert(`BlinkInit error ${e}`);
        }
    }

    class bulbInfo {
        constructor(public name: string, public btn: HTMLButtonElement, public OnColor: string, public OffColor) {
            this.On = <HTMLImageElement>document.getElementById(name + "On");
            this.Off = <HTMLImageElement>document.getElementById(name + "Off");
        }

        set(onOff: boolean) {
            if (onOff == null) return;
            if (onOff) {
                this.Off.style.display = "none";
                this.On.style.display = "inline";
            }
            else {
                this.On.style.display = "none";
                this.Off.style.display = "inline";
            }
        }

        On: HTMLImageElement;
        Off: HTMLImageElement;
    }

    let redInfo: bulbInfo;
    let greenInfo: bulbInfo;

    let spanPeriod: HTMLSpanElement;
    let spanCount: HTMLSpanElement;
    let intervalInput: HTMLInputElement;
    let intval: HTMLSpanElement;

    let lastDay: number;

    function timeStamp() {  // Return time stamp via a quick hack
        let ts = new Date().toString().split(' ');
        //   ['Thu',      0
        //    'Nov',      1
        //    '26',       2
        //    '2015',     3
        //    '22:11:38', 4
        //    'GMT-0500', 5
        //    '(Eastern', 'Standard',
        return ts[4];   // Just the time for now
    }

    function log(msg: string, clr?: string, stamp?: string) {
        let xp = (txt: string, clr?: string) => {
            let pp = document.createElement("p");
            pp.textContent = txt;
            pp.classList.add('log');
            if (clr) pp.style.backgroundColor = clr;
            let fe = logd.firstElementChild;
            logd.insertBefore(pp, fe);
        }

        let dt = new Date();
        if (lastDay != dt.getDate()) {
            lastDay = dt.getDate();
            xp(dt.toString().split(" ").slice(1, 4).join(" "));
        }

        let lines = msg.split('\n');
        stamp = stamp || timeStamp();
        if (lines.length > 1) {
            for (var ln = lines.length - 1; ln >= 0; ln--) {
                let pfx = ln == (lines.length - 1) ? "└" : ln == 0 ? "┌" : "│";
                let st = "\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0"
                if (ln == 0) st = stamp;
                log(`${pfx} ${lines[ln]}`, clr, st);
            }
            return;
        }
        xp(`${stamp} ${msg}`, clr);
    }

    interface Blinker {
        Red?: boolean,
        Green?: boolean
    }

    function Blink(bm: Blinker) {
        //var bm2: BlinkMsgV2 = { Red: TurnOn, Green: !TurnOn };
        let bm2: BlinkMsgV2 = { Red: bm.Red, Green: bm.Green, Interval: parseFloat(intervalInput.value) };
        ws.send(bm2);
    }

    function BlinkHandler(result: string) {
        let resp = document.getElementById("response");
        resp.textContent = result;
        let bm: BlinkMsgV2 = JSON.parse(result);
    }

    class sock {
        constructor() {
            let secure = window.location.protocol.indexOf("https") >= 0;
            let host = window.location.hostname;
            let port = parseInt(window.location.port) + 2; //  secure ? 8004 : 8003;
            let wsx = "ws" + (secure ? "s" : "");

            this.wrl = `${wsx}://${host}:${port}`;
            this.Init();
        }
        wrl: string;
        ws: WebSocket;

        private Init() {
            try {
                log(`Initializing Web Socket ${this.wrl}`);
                this.ws = new WebSocket(this.wrl);
                this.ws.addEventListener("message", this.GotMessage.bind(this));
                this.ws.addEventListener("close", this.Closed.bind(this));
                this.ws.addEventListener("error", this.GotError.bind(this));
                log("Constructed websocket");
            }
            catch (e) {
                log(`Failed to Init ${e}`);
                setTimeout(this.Init.bind(this), 5000);    // Try again in five seconds
            }
        }

        get isOpen() { return (this.ws && this.ws.readyState == WebSocket.OPEN) == true; }
        get isConnecting() { return (this.ws && this.ws.readyState == WebSocket.CONNECTING) == true; }
        get isClosed() { return !this.ws || (this.ws.readyState == WebSocket.CLOSED || this.ws.readyState == WebSocket.CLOSING); }

        public send(msg: string | Object) {
            let txt = typeof msg == "string" ? <string>msg : JSON.stringify(msg);
            try {
                if (!this.isOpen) return;   // For now just ignore until open
                log(`WS Sending "${txt}"`);
                this.ws.send(txt);
            }
            catch (e) {
                //log(`Ws.send error ${e}`);
            }
            return;
        }

        GotMessage(me: MessageEvent) {
            let msg = <string>me.data;
            if (msg[0] == "{") {
                let obj: BlinkReport = JSON.parse(msg);
                redInfo.set(obj.Red);
                greenInfo.set(obj.Green);
                document.getElementById("cname").textContent = obj.CName;
                if (obj.Count) spanCount.textContent = ` Count=${obj.Count}`;
                if (obj.Interval) {
                    let interval = obj.Interval.toString();
                    if (intervalInput.value != interval) {
                        intval.textContent = interval;
                        intval.style.backgroundColor = "yellow";
                        intervalInput.value = interval;
                    }
                }
                spanPeriod.textContent = obj.OnPeriod.toString();
            }
            else {
                if (msg == "reload!") window.location.reload(true);
                log(`Got ${me.data}`);
            }
        }

        Closed() {
            log(`WS Closed\nWill retry periodically`, "yellow");
            setTimeout(this.Init.bind(this), 1000);    // Try again in one second
            this.ws = null;
        }

        GotError(evt) {
            log(`Web Socket error ${evt}`);
        }
    }

    let ws: sock;
    function wsTest() {
        try {
            ws = new sock();
        }
        catch (e) {
            alert(`Web socket error ${e}`);
        }
    }
}

window.addEventListener("load", () => blinks.BlinkInit());
